var AISA = function(selectorOrElement) {
    var elements = [];

    if (!selectorOrElement) {
        throw new Error('Error');
    }

    
    var addElements = function(selectorOrElement) {
        if (typeof selectorOrElement === 'string') {
            var selectedElements = document.querySelectorAll(selectorOrElement);
            elements = elements.concat(Array.from(selectedElements));
        } else if (selectorOrElement instanceof Window || selectorOrElement === window) {
            elements = [window];  
        } else if (selectorOrElement instanceof HTMLElement) {
            elements = [selectorOrElement];
        } else if (selectorOrElement instanceof NodeList || Array.isArray(selectorOrElement)) {
            elements = Array.from(selectorOrElement);
        } else {
            throw new Error('Error');
        }
    };

    addElements(selectorOrElement);

    var forEachElement = function(callback) {
        elements.forEach(function(element) {
           if (element && element instanceof HTMLElement) {
                callback(element);
            }
        });
    };

    
    this.on = function(eventType, callback) {
        forEachElement(function($element) {
            this.addEventListener(eventType, callback);
        });
        return this;
    };

    this.hide = function() {
        elements.forEach(function(element) {
            element.style.display = 'none';
        });
        return this;
    };

    this.show = function() {
        elements.forEach(function(element) {
            element.style.display = 'block';
        });
        return this;
    };

    this.addClass = function(className) {
        elements.forEach(function(element) {
            element.classList.add(className);
        });
        return this;
    };

    this.removeClass = function(className) {
        elements.forEach(function(element) {
            element.classList.remove(className);
        });
        return this;
    };

    this.toggle = function() {
        elements.forEach(function(element) {
            if (element.style.display === 'none') {
                element.style.display = 'block';
            } else {
                element.style.display = 'none';
            }
        });
        return this;
    };

    this.attr = function(attributeName) {
        if (elements.length > 0) {
            return elements[0].getAttribute(attributeName);
        } else {
            return null;
        }
    };

    this.height = function(newHeight) {
        if (elements.length > 0) {
            if (elements[0] instanceof Window) {
                return window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
            } else {
                if (newHeight !== undefined) {
                    elements.forEach(function(element) {
                        element.style.height = newHeight + 'px';
                    });
                    return this;
                } else {
                    return elements[0].offsetHeight;
                }
            }
        } else {
            return null;
        }
    };

    this.width = function(newWidth) {
        if (elements.length > 0) {
            if (newWidth !== undefined) {
                elements.forEach(function(element) {
                    element.style.width = newWidth + 'px';
                });
                return this;
            } else {
                return elements[0].offsetWidth;
            }
        } else {
            return null;
        }
    };

    this.innerHeight = function() {
        if (elements.length > 0) {
	        if (elements[0] instanceof Window) {
	            return window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
	        } else {
	            return elements[0].clientHeight;
	        }
	    } else {
	        return null;
	    }

    };

    this.innerWidth = function() {
        if (elements.length > 0) {
	        if (elements[0] instanceof Window) {
	            return window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
	        } else {
	            return elements[0].clientWidth;
	        }
	    } else {
	        return null;
	    }
    };
    
    this.remove = function() {
       elements.forEach(function(element) {
	        element.remove();
	    });
        return self;
    };
    
    this.html = function(content) {
	    elements.forEach(function(element) {
	        element.innerHTML = content;
	    });
	    return this;
	};
	
	this.css = function(styles) {
    if (arguments.length === 0) {
        
        if (elements.length > 0) {
            var computedStyles = window.getComputedStyle(elements[0]);
            var stylesObject = {};
            for (var i = 0; i < computedStyles.length; i++) {
                var propName = computedStyles[i];
                stylesObject[propName] = computedStyles.getPropertyValue(propName);
            }
            return stylesObject;
        } else {
            return null;
        }
    } else {
        
        elements.forEach(function(element) {
            Object.keys(styles).forEach(function(styleName) {
                element.style[styleName] = styles[styleName];
            });
        });
        return this;
    }
};
    
    
    
	this.val = function(value) {
	    if (typeof value !== 'undefined') {
	        elements.forEach(function(element) {
	            if (element.tagName.toLowerCase() === 'input' || element.tagName.toLowerCase() === 'textarea') {
	                element.value = value;
	            } else if (element.tagName.toLowerCase() === 'select') {
	                var foundOption = false;
	                Array.from(element.options).forEach(function(option) {
	                    if (option.value === value) {
	                        option.selected = true;
	                        foundOption = true;
	                    } else {
	                        option.selected = false;
	                    }
	                });
	                if (!foundOption) {
	                    var newOption = document.createElement('option');
	                    newOption.value = value;
	                    newOption.text = value;
	                    newOption.selected = true;
	                    element.appendChild(newOption);
	                }
	            }
	        });
	        return this;
	    } else {
	        if (elements.length > 0) {
	            var element = elements[0];
	            if (element.tagName.toLowerCase() === 'input' || element.tagName.toLowerCase() === 'textarea') {
	                return element.value;
	            } else if (element.tagName.toLowerCase() === 'select') {
	                var selectedOption = element.querySelector('option:checked');
	                return selectedOption ? selectedOption.value : null;
	            }
	        } else {
	            return null;
	        }
	    }
	};
	
	
	this.submit = function() {
	    elements.forEach(function(element) {
	        if (element.tagName.toLowerCase() === 'form') {
	            element.submit();
	        }
	    });
	    return this;
	};
	
	this.slideToggle = function(duration = 400, callback) {
	    elements.forEach(function(element) {
	        if (window.getComputedStyle(element).display === 'none') {
	            var startHeight = 0;
	            element.style.display = 'block';
	            var endHeight = element.clientHeight;
	            element.style.height = startHeight + 'px';
	            element.style.overflow = 'hidden';
	            var startTime;
	            function slideDown(timestamp) {
	                if (!startTime) startTime = timestamp;
	                var progress = timestamp - startTime;
	                element.style.height = Math.min(progress / duration * endHeight, endHeight) + 'px';
	                if (progress < duration) {
	                    window.requestAnimationFrame(slideDown);
	                } else {
	                    element.style.height = '';
	                    element.style.overflow = '';
	                    if (typeof callback === 'function') {
	                        callback();
	                    }
	                }
	            }
	            window.requestAnimationFrame(slideDown);
	        } else {
	            var startHeight = element.clientHeight;
	            var endHeight = 0;
	            element.style.overflow = 'hidden';
	            var startTime;
	            function slideUp(timestamp) {
	                if (!startTime) startTime = timestamp;
	                var progress = timestamp - startTime;
	                element.style.height = Math.max(startHeight - progress / duration * startHeight, 0) + 'px';
	                if (progress < duration) {
	                    window.requestAnimationFrame(slideUp);
	                } else {
	                    element.style.display = 'none';
	                    element.style.height = '';
	                    element.style.overflow = '';
	                    if (typeof callback === 'function') {
	                        callback();
	                    }
	                }
	            }
	            window.requestAnimationFrame(slideUp);
	        }
	    });
	    return this;
	};

	
	this.minHeight = function(newMinHeight) {
	    if (newMinHeight !== undefined) {
	        forEachElement(function(element) {
	            if (element && element instanceof HTMLElement) {
	                element.style.minHeight = newMinHeight + 'px';
	            }
	        });
	        return this;
	    } else {
	        if (elements.length > 0) {
	            var computedStyle = window.getComputedStyle(elements[0]);
	            return computedStyle.minHeight;
	        } else {
	            return null;
	        }
	    }
	};
	
	
	this.append = function(content) {
        if (typeof content === 'string') {
            elements.forEach(function(element) {
                element.insertAdjacentHTML('beforeend', content);
            });
        } else if (content instanceof HTMLElement) {
            elements.forEach(function(element) {
                element.appendChild(content.cloneNode(true));
            });
        }
        return this;
    };

};

var $ = function(selectorOrElement) {
    return new AISA(selectorOrElement);
};