window.addEventListener('popstate', function(event) {
  loadCodexAjax(location.href);
});


function scrollToTop(duration = 0) {
	window.scrollTo({ top: 0, behavior: 'smooth' });
}
function updatePageMeta(title, description) {
  document.title = title;
  var metaDescription = document.querySelector('meta[name="description"]');
  if (metaDescription) {
    metaDescription.setAttribute('content', description);
  } else {
    metaDescription = document.createElement('meta');
    metaDescription.name = 'description';
    metaDescription.content = description;
    document.head.appendChild(metaDescription);
  }
}

function isElementVisible(element) {
  let currentElement = element;
  while (currentElement) {
    const computedStyle = getComputedStyle(currentElement);
    if (computedStyle.display === 'none' || computedStyle.visibility === 'hidden') {
      return false;
    }
    currentElement = currentElement.parentElement;
  }
  return true;
}


function getFormData(event,formErrTitle,formErrMessage) {
  const formElement = event.target.closest("FORM"); 
  if (!formElement) {
    return null;
  }
  const formData = new FormData(formElement); 
  const formObject = {}; 
  formData.forEach((value, key) => {
    formObject[key] = value;
  });
  const elements = formElement.querySelectorAll('[required]');
  let hasError = false;
  elements.forEach(element => {
    if (element.hasAttribute('required')) {
      const key = element.name;
      const computedStyle = getComputedStyle(element);
      const isVisible = computedStyle.display !== 'none' && computedStyle.visibility !== 'hidden' && isElementVisible(element);
      if (isVisible && (!formObject[key] || formObject[key].trim() === '')  && !element.disabled ) {
        element.style.border = '1px solid red';
        hasError = true;
        $notify(formErrTitle,formErrMessage,'e');
      } else {
        element.style.border = '';
      }
    }
  });
  if (hasError) {
  	return null;
  }
  const urlParams = new URLSearchParams(formObject).toString();
  return urlParams; 
}



function handleAllClicks(event) {
	
	const form = event.target.closest("FORM");
  if (form && form.classList.contains("external")) {
    return;
  }
	
  let clickedLink = null;
  const anchorElement = event.target.closest("A");
  if (typeof playAudio === "function") { playAudio(); }
  if (event.target.closest("FORM") && event.target.type === "submit" ) {
    const formElement = event.target.closest("FORM");
    const formErrMessage = formElement.getAttribute('errMessage');
    const formErrTitle = formElement.getAttribute('errTitle');
    var formData = getFormData(event,formErrTitle,formErrMessage);
	const formAction = formElement.action;
	const url = new URL(formAction);
    const formPath = url.pathname;
    const contentNameID = formElement.getAttribute("contentNameID");
    const contentID = formElement.getAttribute("contentID");
    if ( contentID ) {  contentZone = "#"+contentNameID+"_"+contentID; formData = formData+"&contentID="+contentID; } else { contentZone = "main"; }
    
    if( formData )
    {
    	lockScreen();
        ajax({
		    url: formPath+"/ajax",
		    method: 'POST',
		    data: "apiKey="+apiKey+"&submit=submit&"+formData,
		    success: function (response) {
			    loadResponse(response,contentID);
			    unlockScreen();
			},
		    error: function(xhr, status, error) {
	
		    }
		});
    }
    event.preventDefault();
    event.stopImmediatePropagation();
    
  }
  
  
  if (anchorElement) {  	
    clickedLink = anchorElement.getAttribute("href");
    event.preventDefault(); 
    event.stopImmediatePropagation(); 
  }
  
  if (event.target.closest("[onclick]")) {
    const onclickCode = event.target.closest("[onclick]").getAttribute("onclick");
    const match = onclickCode.match(/window\.location\.href\s*=\s*['"](.*?)['"]/);
    if (match) {
      clickedLink = match[1];
    } else {
    	eval(onclickCode);
    }
    event.preventDefault(); 
    event.stopImmediatePropagation(); 
  }
  
  
  if (clickedLink) {
	if ( clickedLink == '/') { clickedLink = "" ; };
	if ( clickedLink != "" && clickedLink[0] !="/")
	{
	  	window.open(clickedLink,"_blank");
	} else {
		if ( clickedLink == '') { window.history.pushState({}, "", "/" ) } else { window.history.pushState({}, "", clickedLink ) };
	    loadCodexAjax(clickedLink);
	}
  }
}



document.addEventListener("DOMContentLoaded", function() {
  document.addEventListener("click", function(event) {
    handleAllClicks(event);
  }, true); 
  document.addEventListener("submit", function(event) {
    event.preventDefault();  
    event.stopImmediatePropagation();  
  }, true);
});




function loadCodexAjax(clickedLink,push)
{
	if ( push )
	{
		window.history.pushState({}, '', clickedLink.replace('/ajax',''));
	}
	ajax({
	    url: clickedLink+"/ajax",
	    method: 'POST',
	    data: "apiKey="+apiKey,
	    success: function(response) {
			loadResponse(response,"main");
	    	scrollToTop();
	    },
	    error: function(xhr, status, error) {

	    }
	});
}

function reload(clickedLink)
{
	loadCodexAjax(clickedLink,'1');
	
}



function loadResponse(response,id)
{
	if ( id == null ) { id = "main"; }
	$(id).html(response);
	var scripts = response.match(/<script[^>]*>([\s\S]*?)<\/script>/g);
	if (scripts) {
        scripts.forEach(function (script) {
            var srcMatch = script.match(/src=["'](.*?)["']/);
            if (srcMatch) {
                var src = srcMatch[1];
                    var newScript = document.createElement('script');
                    newScript.type = 'text/javascript';
                    newScript.src = src;
                    newScript.onload = function () {
                        if (typeof initializeCodexScript === "function") {
                            initializeCodexScript();
                        }
                    };
                    document.head.appendChild(newScript);                
            } else {
                var scriptContent = script.match(/<script(?![^>]*type="application\/ld\+json")[^>]*>.*?<\/script>/gs);
				if (scriptContent && scriptContent.length > 0) {
				    scriptContent.forEach(function(content) {
				        var newInlineScript = document.createElement('script');
				        newInlineScript.type = 'text/javascript';
				        newInlineScript.text = content.replace(/<script[^>]*>|<\/script>/g, ''); // Elimin? tagurile <script> din con?inut
				        document.body.appendChild(newInlineScript);
				    });
				}
            }
        });
    }
}